#!/usr/bin/perl
use strict;
#############################################################################
#
# ppplag.pl -- PPP log analyzer grid :)
# Vladi Belperchinov-Shabanski
# <cade -AT- biscom -DOT- net> <cade -AT- datamax -DOT- bg>
# http://www.biscom.net/~cade
# Any notes, comments, suggestions are welcome! Thank you!
#
# ChangeLog/History:
#
# 04.Nov.2000 -- initial release
# 09.Nov.2001 -- adsl pppd messages support
# 20.Nov.2001 -- filters only pppd log lines
# 09.Dec.2001 -- multi-files (+.gz), -s, -b, -m options(s)
# 04.Jan.2002 -- lock on pid, -i option(s)
# 06.May.2003 -- -a and -e options
# 09.May.2003 -- -u option
#                thanx to Robert Hoekstra
#                <robert -DOT- hoekstra -AT- parox -DOT- nl> for this patch
#                -l option added
# 11.May.2003 -- PPPLAG_LOGFILE env var added
#
#############################################################################
# config section -- edit as you want/need
#############################################################################

my $VERSION = '$Id: ppplag.pl,v 1.10 2003/05/11 12:05:45 cade Exp $';

my $opt_sumonly = 0;       # show only summary?
my $opt_summon = 0;        # show month summaries only?
my $opt_nobio = 0;         # don't show bps/in/out?
my $opt_auth_user_in = 0;  # show auth user in (server-side)?
my $opt_auth_user_out = 0; # show auth user out (client-side)?
my $opt_if = "--";         # interface? ppp0, ppp1, ttyS0, ttyS1, etc
                           # this will match anytime `ppp0 <--> /dev/modem'

my $opt_begin_after;
my $opt_end_after;

#############################################################################
# config section end -- don't edit below...
# everything below this line is just code -- nothing much to see there :)
#############################################################################

if( $ARGV[0] =~ /-?-h(elp)?/i )
  {
  die "ppplag.pl by Vladi Belperchinov-Shabanski 'Cade' 2001\n$VERSION\n" .
      "usage: ppplag.pl [options] logfile...\n" .
      "options:\n" .
      "   -s       show summary only\n" .
      "   -u       show auth-in user (server-side pppd logs)\n" .
      "   -l       show auth-out user (client-side pppd logs)\n" .
      "            (this requires pppd `debug' messages!)\n" .
      "   -b       don't show bps/in/out\n" .
      "   -m       show summaries by month\n" .
      "   -i if    use specific interface (ppp0,ttyS1,eth0,etc...)\n" .
      "   -a str   begin stats after `str' found\n" .
      "   -e str   end   stats after `str' found\n" .
      "            example: -a \"Apr 10 11:11\"\n" .
      "            example: -e \"Mar 12 12:12\"\n" .
      "";
  }

my @LOG;
 
while( $_ = shift @ARGV )
  {
  $opt_sumonly = 1 if /^-s/;
  $opt_nobio = 1 if /^-b/;
  $opt_summon = 1 if /^-m/;
  $opt_auth_user_in = 1 if /^-u/;
  $opt_auth_user_out = 1 if /^-l/;
  $opt_if = shift @ARGV if /^-i/;
  $opt_begin_after = shift @ARGV if /^-a/;
  $opt_end_after = shift @ARGV if /^-e/;
  next if /^-/;
  push @LOG, $_;
  }

push @LOG, ( $ENV{ PPPLAG_LOGFILE } || '/var/log/messages' ) unless @LOG;

if ( $opt_sumonly or $opt_summon )
  {
  $opt_auth_user_in = 0;
  $opt_auth_user_out = 0;
  }

my @logformat = ( 12, 0, 0, 6, 14, 14, 8, 8 );

$logformat[1] = 10 if $opt_auth_user_in;  # user auth-in  (server-side logs)
$logformat[2] = 10 if $opt_auth_user_out; # user auth-out (client-side logs)
$logformat[6] =  0 if $opt_nobio; # hide bps
$logformat[7] =  0 if $opt_nobio; # hide bps

for my $log ( @LOG )
  {
  process_log( $log );
  }

#############################################################################
# process log sub
#############################################################################

my $best_speed_in  = 0;
my $best_speed_out = 0;

sub process_log
{
  my $log = shift;
  if ( ! -r $log )
    {
    print "ppplag.pl: cannot read $log, cannot continue\n";
    exit;
    }
 
  my $sep = "-" x 79 . "\n";
 
  my $total_time = 0;
  my $total_in   = 0;
  my $total_out  = 0;
 
  my $best_time  = 0;
  my $best_in    = 0;
  my $best_out   = 0;
 
  my $last_mon  = '';
  my $mon_time  = 0;
  my $mon_in    = 0;
  my $mon_out   = 0;
 
  a_printf( \@logformat, "Start Time", "User IN", "User OUT",  "HH:MM", "Bytes IN", "Bytes OUT", "Bps/IN", "Bps/OUT" );
 
  print $sep unless $opt_sumonly;

  my $inside = defined $opt_begin_after ? 0 : 1;
 
  $_ = $log;
  open( my $i, /\.gz$/ ? "gzip -dc $log|" : "$log" );
  MAINLOOP:
  while(<$i>)
    {
      if ( /$opt_if/ and /pppd\[(\d+)\]: +Connect: +/ )
        {
      if( defined $opt_begin_after )
        {
        $inside = 1 if index( $_, $opt_begin_after ) > -1;
        next unless $inside;
        }
      if( defined $opt_end_after )
        {
        close $i if index( $_, $opt_end_after ) > -1;
        }
 
      my $ppid = $1;
 
      my $this_mon;
      my $this_start;
      my $this_time = 0;
      my $this_in   = 0;
      my $this_out  = 0;
      my $user_in  = 'n/a'; # server-side user (auth-in)
      my $user_out = 'n/a'; # client-side user (auth-out) FIXME: pap only?
 
      ( $this_start, $this_mon ) = ( $1, $2 ) if /^((...) .. ..:..):../;
      $last_mon = $this_mon unless $last_mon;

      while(<$i>)
        {
        redo MAINLOOP if /$opt_if/ and /pppd\[(\d+)\]: +Connect: +/;
        next unless /pppd\[$ppid\]/;
        # last if /Exit\./ or /Doing disconnect/;
        $user_in = $1 if /peer authentication succeeded for (.*)/;
        $user_out = $1 if /user="([^\"]+)"/;
        $this_time = $1 if /Connect time ([\d\.]+) minutes\./;
        if ( /Sent (\d+) bytes, received (\d+) bytes\./ )
          {
          $this_out = $1;
          $this_in  = $2;
          last;
          };
        }
      next if $this_time == 0;
 
      if( $last_mon && $last_mon ne $this_mon )
        {
        print_line( $last_mon, $user_in, $user_out, $mon_time, $mon_in, $mon_out)
          if !$opt_sumonly and $opt_summon;
 
        $last_mon = $this_mon;
        $mon_time = 0;
        $mon_in = 0;
        $mon_out = 0;
        }
 
      print_line( $this_start, $user_in, $user_out, $this_time, $this_in, $this_out)
        unless $opt_sumonly or $opt_summon;
 
      $best_time  = $this_time  if $this_time > $best_time;
      $best_in    = $this_in    if $this_in   > $best_in;
      $best_out   = $this_out   if $this_out  > $best_out;
 
      $mon_time   += $this_time;
      $mon_in   += $this_in;
      $mon_out  += $this_out;
 
      $total_time += $this_time;
      $total_in   += $this_in;
      $total_out  += $this_out;
      }
 
    }
  print_line( $last_mon, "", "", $mon_time, $mon_in, $mon_out)
    if !$opt_sumonly and $opt_summon;
  close $i;
  print $sep;
 
  print_line( "Best:",  "", "", $best_time, $best_in, $best_out );
  print_line( "Total:", "", "", $total_time, $total_in, $total_out );
 
  print $sep;
}

sub print_line
{
  my $_label = shift;
  my $_user_in = shift;
  my $_user_out = shift;
  my $_time = shift;
  my $_in = shift;
  my $_out = shift;
 
  my $_time2 = sprintf "%s:%02s", int($_time / 60), int($_time % 60);
  my $_speed_in  = $_time > 0 ? int(( $_in  ) / ( $_time * 60 )) : 'n/a';
  my $_speed_out = $_time > 0 ? int(( $_out ) / ( $_time * 60 )) : 'n/a';

  $best_speed_in  = $_speed_in  if $_speed_in  > $best_speed_in ;
  $best_speed_out = $_speed_out if $_speed_out > $best_speed_out;

  a_printf( \@logformat, $_label, $_user_in, $_user_out, $_time2, comma($_in), comma($_out), $_speed_in, $_speed_out );
}

#############################################################################
# utility functions
#############################################################################

sub a_printf
{
  my $ar = shift;
  my @a = @$ar;
  my $s;
  for( @_ )
    {
    my $n = shift @a;
    next unless $n > 0;
    my $__ = substr( $_, 0, $n );
    $s .= sprintf "%" . $n . "s ", $__;
    }
  print $s . "\n";
}

sub comma
{
  my $res = reverse shift;
  $res =~ s/(\d\d\d)/$1,/go;
  $res =~ s/,$//;
  $res = reverse $res;
}

#############################################################################
# eof
#############################################################################

